/*
 * ShowOff Widget 1.25
 * ©2005-2007 Joakim NygÂrd, all rights reserved
 * fetchAverages() contributed by Oliver Sprenger
 * Swedish localization by Fredrik Olsson
 * Finnish and Japanese localization by Juho Viitasalo
 * Polish localization by Cezary Okupski
 *
 * This Widget is provided as is with no warranty and
 * no guarantees.
 * You may freely distribute this software without modifications.
 * If you modify this Widget, please credit me and send me a copy.
 * If you read this far, have fun and be inspired :)
 */

// defines the different display modes.
var currentDisplay = 0;

// this holds the record uptime
var recordKey = 0;
var timer = null;


function load()
{
	setupParts();
}

function remove()
{
	// your widget has just been removed from the layer
	// remove any preferences as needed
	// widget.setPreferenceForKey(null, createInstancePreferenceKey("your-key"));
}

// toggle between the display modes
function toggleDisplay() {
	// we have 4 settings, loop through them
	currentDisplay = ++currentDisplay % 4;
	
	// don't call onshow which updates the timer,
	// just updateDisplay to reflect the changes.
	updateDisplay();
}

// open Safari and go to the url given
function goToUrl(url)
{
	if (window.widget)
	{
		widget.openURL(url);
	}
}

function hide()
{
	// your widget has just been hidden stop any timers to
	// prevent cpu usage
	var currentUp = fetchRecord();
	savePrefs(currentUp);
	if (timer != null) {
	    clearInterval(timer);
	    timer = null;
	}
	if (timer == null) {
		// update the display every 2 minutes
		timer = setInterval("backgroundSave();", 240000);
	}
}

function show()
{
	// your widget has just been shown.  restart any timers
	// and adjust your interface as needed
	readPrefs();

	if (timer != null) {
	    clearInterval(timer);
	    timer = null;
	}	
	if (timer == null) {
		updateDisplay();
		// update the display every 2 seconds
		timer = setInterval("updateDisplay();", 2000);
	}
}

// Display the info requested as set by the currentDisplay variable
// Called every 2 seconds
function updateDisplay() {
	// uptime
	if(currentDisplay==0) {
		document.getElementById("uptime").innerText = readable(fetchUptime());
		document.getElementById("title").innerText = getLocalizedString("Uptime");
	} 
	// record
	else if(currentDisplay==1) {
		document.getElementById("uptime").innerText = readable(fetchRecord());
		document.getElementById("title").innerText = getLocalizedString("Record");
	}
	// load
	else if(currentDisplay==2) {
		document.getElementById("uptime").innerText = fetchAverages();
		document.getElementById("title").innerText = getLocalizedString("Load");	
	}
	// vm usage
	else {
		document.getElementById("uptime").innerText = fetchVMUsage();
		document.getElementById("title").innerText = getLocalizedString("Swap");
	}
}

// method contributed by Oliver Sprenger, dev@sprenger-mail.de
function fetchAverages() {
    outString = widget.system("/usr/bin/uptime", null).outputString;
    return outString.substring(outString.indexOf('averages')+10);
}

// get the VM file usage
function fetchVMUsage() {
	outString = widget.system("/usr/sbin/sysctl -n vm.swapusage", null).outputString;
	var parts = outString.split('  ');
	var total = parts[0].substring(parts[0].indexOf('=')+2,parts[0].indexOf('M')-3)+"MB";
	var usage = parts[1].substring(parts[1].indexOf('=')+2,parts[1].indexOf('M')-3)+"MB";
	var free  = parts[2].substring(parts[2].indexOf('=')+2,parts[2].indexOf('M')-3)+"MB";
	return usage+"/"+total;
}

function fetchUptime() {
	//debug value;
	//outString = "18:44  up 21 mins, 8 users, load averages: 1.61 1.76 1.67";
	outString = widget.system("/usr/bin/uptime", null).outputString;
	var upString = outString.substring(0,outString.indexOf("load")-2);
	upString = upString.substring(upString.indexOf("up")+3,upString.lastIndexOf(","));
	return getMinutes(upString.split(","));
}

function fetchRecord() {
	var currentUp = fetchUptime();
	
	// is old record still valid?
	if(recordKey != 0 && recordKey > currentUp) {
	    return recordKey;
	}
	// else new record
	else {
		return currentUp;
	}
}

// method contributed by Oliver Sprenger, dev@sprenger-mail.de
function readable(string) {
	var tmp = string % 1440;
	var days = (string-(string % 1440))/1440; 
	var hours = (tmp-(tmp % 60))/60; 
	var minute = tmp % 60;
	var verboseString = "";
	
	if(days > 0) {
		if(days == 1)
			verboseString += days+ getLocalizedString(' day');
		else
			verboseString += days+ getLocalizedString(' days');
	}
		
	if(hours > 0) {
		if (verboseString != "")
		   verboseString += ", ";
		   
		if(hours < 10)
			verboseString += "0";
		verboseString += hours+":";
		
		if(minute < 10)
			verboseString += "0";
		verboseString += minute+"";
	}
    else if (minute > 0)
	{
		if (verboseString != "")
		   verboseString += ", ";
	
		if(minute < 10)
			verboseString += "0";
		verboseString += minute;
		
		if (minute == 1)
			verboseString += getLocalizedString(' minute');
		else if (typeof localizedStrings[' minutes234'] != 'undefined' && (minute % 10 == 2 || minute % 10 == 3 || minute % 10 == 4) && (minute - minute % 10) % 100 != 10)
			verboseString += getLocalizedString(' minutes234');
		else
			verboseString += getLocalizedString(' minutes');
	}
	return verboseString;
}

function getMinutes(string) {
	var currentUp = 0;
	for(i=string.length-1;i>=0;i--) {
		// minutes and hours
		if(i==string.length-1) {
			var timePart = string[i].split(":");
			// we have both hours and minutes
			if(timePart.length > 1) {
				currentUp = Number(timePart[1]);
				currentUp += Number(timePart[0]*60);
			}
			// either less than one hour or a whole amount of hours
			else {
				timePart = string[i].split(" ");
				// remove extra array element
				if(timePart[0]=="") {
					timePart = timePart.slice(1);
				}
				if(timePart[1]=="hrs" || timePart[1]=="hours" || timePart[1]=="hour") {
					currentUp = Number(timePart[0]*60);
				} else if(timePart[1]=="min" || timePart[1]=="mins") {
					currentUp = Number(timePart[0]);
				}
			}
		}
		// we have days
		else if(i==string.length-2) {
			timePart = string[i].split(" ");
			currentUp += Number(timePart[0]*24*60);
		}
		// reaching weeks
		else if(i==string.length-3) {
			timePart = string[i].split(" ");
			currentUp += Number(timePart[0]*7*24*60);
		}
	}
	return currentUp;
}

function readPrefs() {
	if(window.widget) {
		recordKey = widget.preferenceForKey("record");
		if (isNaN(recordKey))
		{
			recordKey = 0;
		}
	}
}

function backgroundSave() {
	var currentUp = fetchRecord();
	savePrefs(currentUp);
}

function savePrefs(soFar) {
	if(soFar > recordKey) {
		if(window.widget) {
			widget.setPreferenceForKey(soFar, "record");
		}
	}
}

function getLocalizedString (key)
{
	try {
		var ret = localizedStrings[key];
		if (ret === undefined)
			ret = key;
		return ret;
	} catch (ex) {}

	return key;
}

function showBack(event)
{
	// your widget needs to show the back

	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToBack");

	front.style.display="none";
	back.style.display="block";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}

function showFront(event)
{
	// your widget needs to show the front

	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");

	front.style.display="block";
	back.style.display="none";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}

if (window.widget)
{
	widget.onremove = remove;
	widget.onhide = hide;
	widget.onshow = show;
}
